import { max, min } from 'd3-array';
import { scaleOrdinal, scaleLinear, scaleLog, scaleTime, } from 'd3-scale';
import { createContext, useContext, useMemo } from 'react';
import { validatePosition } from '../utils';
export function plotReducer(state, action) {
    switch (action.type) {
        case 'newData': {
            state.series.push(action.payload);
            break;
        }
        case 'removeData': {
            const { id } = action.payload;
            const seriesFiltered = state.series.filter((series) => series.id !== id);
            state.series = seriesFiltered;
            break;
        }
        case 'newAxis': {
            const { id, position, ...values } = action.payload;
            let currentAxis = state.axes[id];
            if (currentAxis) {
                validatePosition(currentAxis.position, position, id);
                state.axes[id] = { ...currentAxis, position, ...values };
            }
            else {
                state.axes[id] = { position, ...values };
            }
            break;
        }
        case 'removeAxis': {
            const { id } = action.payload;
            delete state.axes[id];
            break;
        }
        case 'addHeading': {
            state.headingPosition = action.payload.position;
            break;
        }
        case 'removeHeading': {
            state.headingPosition = null;
            break;
        }
        case 'addLegend': {
            state.legendPosition = action.payload.position;
            state.legendMargin = action.payload.margin;
            break;
        }
        case 'removeLegend': {
            state.legendPosition = null;
            state.legendMargin = 0;
            break;
        }
        default: {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            throw new Error(`Unknown reducer type ${action.type}`);
        }
    }
}
export const plotContext = createContext({
    width: 0,
    height: 0,
    plotWidth: 0,
    plotHeight: 0,
    colorScaler: scaleOrdinal(),
    axisContext: {},
});
export const plotDispatchContext = createContext(() => {
    // No-op
});
export function usePlotContext() {
    const context = useContext(plotContext);
    if (!context) {
        throw new Error('usePlotContext called outside of Plot context');
    }
    return context;
}
export function usePlotDispatchContext() {
    const context = useContext(plotDispatchContext);
    if (!context) {
        throw new Error('usePlotDispatchContext called outside of Plot context');
    }
    return context;
}
export function useAxisContext(state, axesOverrides, { plotWidth, plotHeight }) {
    const context = useMemo(() => {
        let axisContext = {};
        for (const id in state.axes) {
            const axis = state.axes[id];
            const overrides = axesOverrides[id];
            const isHorizontal = ['top', 'bottom'].includes(axis.position);
            const xY = isHorizontal ? 'x' : 'y';
            // Get limits from override (context), state (axis props), or data.
            const axisMin = (overrides === null || overrides === void 0 ? void 0 : overrides.min) !== undefined
                ? overrides.min
                : axis.min !== undefined
                    ? axis.min
                    : min(state.series, (d) => d[xY].min);
            const axisMax = (overrides === null || overrides === void 0 ? void 0 : overrides.max) !== undefined
                ? overrides.max
                : axis.max !== undefined
                    ? axis.max
                    : max(state.series, (d) => d[xY].max);
            // Limits validation
            if (axisMin === undefined || axisMax === undefined) {
                return {};
            }
            if (axisMin > axisMax) {
                throw new Error(`${id}: min (${axisMin}) is bigger than max (${axisMax})`);
            }
            // Limits paddings
            const diff = axisMax - axisMin;
            const minPad = diff * (axis.paddingStart || 0);
            const maxPad = diff * (axis.paddingEnd || 0);
            const range = isHorizontal ? [0, plotWidth] : [plotHeight, 0];
            const domain = [axisMin - minPad, axisMax + maxPad];
            const clampInDomain = function clampInDomain(value) {
                return value < domain[0]
                    ? domain[0]
                    : value > domain[1]
                        ? domain[1]
                        : value;
            };
            switch (axis.scale) {
                case 'log': {
                    axisContext[id] = {
                        type: axis.scale,
                        position: axis.position,
                        tickLabelFormat: axis.tickLabelFormat,
                        scale: scaleLog()
                            .domain(domain)
                            .range(axis.flip ? range.reverse() : range),
                        domain,
                        clampInDomain,
                    };
                    break;
                }
                case 'time': {
                    axisContext[id] = {
                        type: axis.scale,
                        position: axis.position,
                        tickLabelFormat: axis.tickLabelFormat,
                        scale: scaleTime()
                            .domain(domain)
                            .range(axis.flip ? range.reverse() : range),
                        domain,
                        clampInDomain,
                    };
                    break;
                }
                case 'linear':
                default: {
                    axisContext[id] = {
                        type: 'linear',
                        position: axis.position,
                        tickLabelFormat: axis.tickLabelFormat,
                        scale: scaleLinear()
                            .domain(domain)
                            .range(axis.flip ? range.reverse() : range),
                        domain,
                        clampInDomain,
                    };
                    break;
                }
            }
        }
        return axisContext;
    }, [state.axes, state.series, axesOverrides, plotWidth, plotHeight]);
    return context;
}
//# sourceMappingURL=plotContext.js.map