"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const ml_distance_euclidean_1 = require("ml-distance-euclidean");
const react_1 = require("react");
const utils_1 = require("../utils");
const HORIZONTAL = ['bottom', 'top'];
// function infoFromWheel(
//   event: React.WheelEvent<SVGRectElement>,
//   axisContext: Record<string, PlotAxisContext>,
//   plotHeight: number,
// ) {
//   const ratio = 1 + event.deltaY * -0.001;
//   // Calculate coordinates
//   let coordinates: TrackingResult['coordinates'] = {};
//   const { scale } = axisContext.y;
//   const yPosition = scale(0);
//   const y1 =
//     ratio > 1 ? yPosition * (1 - 1 / ratio) : yPosition * (1 - 1 / ratio);
//   const y2 =
//     ratio > 1
//       ? yPosition + (plotHeight - yPosition) / ratio
//       : plotHeight + (plotHeight - yPosition) * (1 / ratio - 1);
//   coordinates.y1 = toNumber(scale.invert(y1));
//   coordinates.y2 = toNumber(scale.invert(y2));
//   return {
//     event,
//     coordinates,
//   };
// }
function infoFromMouse(event, axisContext, stateSeries, target) {
    const { clientX, clientY, movementX, movementY } = event;
    const { left, top } = target.getBoundingClientRect();
    // Calculate coordinates
    const xPosition = clientX - left;
    const yPosition = clientY - top;
    const coordinates = {};
    const clampedCoordinates = {};
    const movement = {};
    for (const key in axisContext) {
        const { scale, clampInDomain, position } = axisContext[key];
        if (HORIZONTAL.includes(position)) {
            coordinates[key] = (0, utils_1.toNumber)(scale.invert(xPosition));
            movement[key] =
                (0, utils_1.toNumber)(scale.invert(movementX)) - (0, utils_1.toNumber)(scale.invert(0));
        }
        else {
            coordinates[key] = (0, utils_1.toNumber)(scale.invert(yPosition));
            movement[key] =
                (0, utils_1.toNumber)(scale.invert(movementY)) - (0, utils_1.toNumber)(scale.invert(0));
        }
        clampedCoordinates[key] = clampInDomain(coordinates[key]);
    }
    return {
        event,
        coordinates,
        clampedCoordinates,
        movement,
        getClosest: (method) => closestCalculation(method, { x: xPosition, y: yPosition }, stateSeries, axisContext),
    };
}
function closestCalculation(method, coordinates, stateSeries, axisContext) {
    let series = {};
    switch (method) {
        case 'x': {
            for (const { id, x, data, label } of stateSeries) {
                if (data) {
                    const point = (0, utils_1.closestPoint)(data, coordinates, (point, pos) => {
                        const { scale } = axisContext[x.axisId];
                        const xVal = pos[x.axisId];
                        return Math.abs(scale(point.x) - xVal);
                    });
                    series[id] = { point, label, axis: axisContext[x.axisId] };
                }
            }
            break;
        }
        case 'y': {
            for (const { id, y, data, label } of stateSeries) {
                if (data) {
                    const point = (0, utils_1.closestPoint)(data, coordinates, (point, pos) => {
                        const { scale } = axisContext[y.axisId];
                        const yVal = pos[y.axisId];
                        return Math.abs(scale(point.y) - yVal);
                    });
                    series[id] = { point, label, axis: axisContext[y.axisId] };
                }
            }
            break;
        }
        case 'euclidean': {
            for (const { id, x, y, data, label } of stateSeries) {
                if (data) {
                    const point = (0, utils_1.closestPoint)(data, coordinates, (point, pos) => {
                        const { scale: xScale } = axisContext[x.axisId];
                        const { scale: yScale } = axisContext[y.axisId];
                        const xVal = pos[x.axisId];
                        const yVal = pos[y.axisId];
                        return (0, ml_distance_euclidean_1.euclidean)([xScale(point.x), yScale(point.y)], [xVal, yVal]);
                    });
                    series[id] = {
                        point,
                        label,
                        axis: { x: axisContext[x.axisId], y: axisContext[y.axisId] },
                    };
                }
            }
            break;
        }
        default: {
            throw new Error(`Unknown distance name: ${method}`);
        }
    }
    return series;
}
const mouseEventMap = {
    mouseenter: 'onMouseEnter',
    mouseleave: 'onMouseLeave',
    mousedown: 'onMouseDown',
    mouseup: 'onMouseUp',
    mousemove: 'onMouseMove',
    click: 'onClick',
    dblclick: 'onDoubleClick',
    wheel: 'onWheel',
};
const mouseEvents = [
    'mouseenter',
    'mouseleave',
    'mousedown',
    'mouseup',
    'mousemove',
    'click',
    'dblclick',
    'wheel',
];
const globalMouseEvents = [
    'mousemove',
    'mouseup',
];
function Tracking({ plotId, plotEvents, stateSeries, axisContext, plotHeight, plotWidth, }) {
    const rectRef = (0, react_1.useRef)(null);
    const plotDataRef = (0, react_1.useRef)({
        plotId,
        plotEvents,
        stateSeries,
        axisContext,
        plotHeight,
        plotWidth,
    });
    (0, react_1.useEffect)(() => {
        plotDataRef.current = {
            plotId,
            plotEvents,
            stateSeries,
            axisContext,
            plotHeight,
            plotWidth,
        };
    }, [axisContext, plotEvents, plotHeight, plotId, plotWidth, stateSeries]);
    (0, react_1.useEffect)(() => {
        const rect = rectRef.current;
        if (!rectRef)
            return;
        function mouseEventListener(event) {
            if (event.type === 'mousedown') {
                globalMouseEvents.forEach((mouseEvent) => window.addEventListener(mouseEvent, mouseEventListener));
            }
            else if (event.type === 'mouseup') {
                globalMouseEvents.forEach((mouseEvent) => window.removeEventListener(mouseEvent, mouseEventListener));
            }
            const info = infoFromMouse(event, plotDataRef.current.axisContext, plotDataRef.current.stateSeries, rect);
            plotEvents.handleEvent(plotId, mouseEventMap[event.type], info);
        }
        mouseEvents.forEach((mouseEvent) => rect.addEventListener(mouseEvent, mouseEventListener));
        return () => {
            mouseEvents.forEach((mouseEvent) => rect.removeEventListener(mouseEvent, mouseEventListener));
            globalMouseEvents.forEach((mouseEvent) => window.removeEventListener(mouseEvent, mouseEventListener));
        };
    }, [plotId, plotEvents]);
    return ((0, jsx_runtime_1.jsx)("rect", { ref: rectRef, width: plotWidth, height: plotHeight, style: { fillOpacity: 0, outline: 'none' } }, void 0));
}
exports.default = Tracking;
